/**
 * SimpleMonitor main file.
 *
 * @author Viktoras Agejevas
 * @copyright Viktoras Agejevas 2005 - 2009
 */

include("resources/js/Graph.js");
include("resources/js/Presets.js");

// default visual preferences are from Reborn style preset
preset = eval("simpleMonitorPresets.Reborn");
preferences.bgOpacity.defaultValue = preset.bgOpacity;
preferences.bgColor.defaultValue = preset.bgColor;
preferences.fontFamily.defaultValue = preset.fontFamily;
preferences.textColor.defaultValue = preset.textColor;
preferences.fontSize.defaultValue = preset.fontSize;
preferences.fontWeight.defaultValue = preset.fontWeight;
preferences.graphTextColor.defaultValue = preset.graphTextColor;
preferences.cpuGraphColor.defaultValue = preset.cpuGraphColor;
preferences.ramGraphColor.defaultValue = preset.ramGraphColor;
preferences.netInGraphColor.defaultValue = preset.netInGraphColor;
preferences.netOutGraphColor.defaultValue = preset.netOutGraphColor;

var simpleMonitor;
var server;

var ipCheckRetry = 0;
var ipCheckUrls = new Array();
ipCheckUrls.push("http://visivardai.com/ip.php");
ipCheckUrls.push("http://www.checkip.org/");
ipCheckUrls.push("http://checkip.dyndns.org/");
	
// System info
var trafficIn;
var trafficOut;
var uptime;
var processorName;
var processorSpeed;
var processorMaxSpeed;

// Traffic history
var lastIn = 0;
var lastOut = 0;
var longIn = 0;
var longOut = 0;

// Graphs
var color1 = preferences.cpuGraphColor.value;
var color2 = preferences.ramGraphColor.value;
var color3 = preferences.netInGraphColor.value;
var color4 = preferences.netOutGraphColor.value;

var cpuGraph = new Graph(25, 100, 21, 55, color1);
var ramGraph = new Graph(25, 100, 21, 100, color2);
var netInGraph = new Graph(25, 100, 21, 145, color3);
var netOutGraph = new Graph(25, 100, 21, 190, color4);

var cpuPanel = new Image();
var cpu = new Text();
var cpuSpeedShadow = new Text();
var cpuSpeed = new Text();
var ramPanel = new Image();
var ram = new Text();
var usedRamShadow = new Text();
var usedRam = new Text();
var netInPanel = new Image();
var netIn = new Text();
var netInTotalShadow = new Text();
var netInTotal = new Text();
var netOutPanel = new Image();
var netOut = new Text();
var netOutTotalShadow = new Text();
var netOutTotal = new Text();
var ipAddress = new Text();
var upFor = new Text();

init();
setPrefs();
updateCpuInfo(true);
update();

smWindow.visible = true;

function init() {
	validatePlatform();
	createSimpleMonitorCOM();
	createServerCOM();
	
	cpuPanel.src = "resources/images/panel.png";
	cpuPanel.hOffset = 20;
	cpuPanel.vOffset = 56;
	cpuPanel.vAlign = "bottom";
	cpuPanel.height = 27;
	cpuPanel.width = 102;
	cpuPanel.opacity = 30;

	cpu.alignment = "left";
	cpu.vOffset = 27;
	cpu.hOffset = 25;
	
	cpuSpeedShadow.vOffset = 40;
	cpuSpeedShadow.hOffset = 71;
	cpuSpeedShadow.opacity = 180;
	cpuSpeedShadow.hAlign = "center";
	
	cpuSpeed.vOffset = 39;
	cpuSpeed.hOffset = 70;
	cpuSpeed.opacity = 180;
	cpuSpeed.hAlign = "center";

	ramPanel.src = "resources/images/panel.png";
	ramPanel.hOffset = 20;
	ramPanel.vOffset = 101;
	ramPanel.vAlign = "bottom";
	ramPanel.height = 27;
	ramPanel.width = 102;
	ramPanel.opacity = 30;

	ram.alignment = "left";
	ram.vOffset = 72;
	ram.hOffset = 25;
	
	usedRamShadow.vOffset = 85;
	usedRamShadow.hOffset = 71;
	usedRamShadow.style = "bold";
	usedRamShadow.hAlign = "center";

	usedRam.vOffset = 84;
	usedRam.hOffset = 70;
	usedRam.style = "bold";
	usedRam.opacity = 180;
	usedRam.hAlign = "center";

	netInPanel.src = "resources/images/panel.png";
	netInPanel.hOffset = 20;
	netInPanel.vOffset = 146;
	netInPanel.vAlign = "bottom";
	netInPanel.height = 27;
	netInPanel.width = 102;
	netInPanel.opacity = 30;

	netIn.alignment = "left";
	netIn.vOffset = 117;
	netIn.hOffset = 25;

	netInTotalShadow.vOffset = 130;
	netInTotalShadow.hOffset = 71;
	netInTotalShadow.style = "bold";
	netInTotalShadow.hAlign = "center";
	
	netInTotal.vOffset = 129;
	netInTotal.hOffset = 70;
	netInTotal.style = "bold";
	netInTotal.opacity = 180;
	netInTotal.hAlign = "center";

	netOutPanel.src = "resources/images/panel.png";
	netOutPanel.hOffset = 20;
	netOutPanel.vOffset = 191;
	netOutPanel.vAlign = "bottom";
	netOutPanel.height = 27;
	netOutPanel.width = 102;
	netOutPanel.opacity = 30;

	netOut.alignment = "left";
	netOut.vOffset = 162;
	netOut.hOffset = 25;

	netOutTotalShadow.vOffset = 175;
	netOutTotalShadow.hOffset = 71;
	netOutTotalShadow.style = "bold";
	netOutTotalShadow.hAlign = "center";

	netOutTotal.vOffset = 174;
	netOutTotal.hOffset = 70;
	netOutTotal.style = "bold";
	netOutTotal.opacity = 180;
	netOutTotal.hAlign = "center";
	
	ipAddress.vOffset = 210;
	ipAddress.hOffset = 70;
	ipAddress.hAlign = "center";
	ipAddress.onMouseEnter = function() {
		ipAddress.opacity = 220;
	};
	ipAddress.onMouseExit = function() {
		ipAddress.opacity = 255;
	};
	ipAddress.onMouseDown = function () {
		preferences.showLocalIp.value = preferences.showLocalIp.value == 1 ? 0 : 1;
		updateIp();
	};
	
	upFor.vOffset = 222;
	upFor.hOffset = 70;
	upFor.hAlign = "center";

	var frame = smWindow.firstChild;

	// panels
	frame.appendChild(cpuPanel);
	frame.appendChild(ramPanel);
	frame.appendChild(netInPanel);
	frame.appendChild(netOutPanel);
	
	// graphs
	frame.appendChild(cpuGraph.getFrame());
	frame.appendChild(ramGraph.getFrame());
	frame.appendChild(netInGraph.getFrame());
	frame.appendChild(netOutGraph.getFrame());
	
	frame.appendChild(cpu);
	frame.appendChild(cpuSpeedShadow);
	frame.appendChild(cpuSpeed);
	
	frame.appendChild(ram);
	frame.appendChild(usedRamShadow);
	frame.appendChild(usedRam);
	
	frame.appendChild(netIn);
	frame.appendChild(netInTotalShadow);
	frame.appendChild(netInTotal);
	
	frame.appendChild(netOut);
	frame.appendChild(netOutTotalShadow);
	frame.appendChild(netOutTotal);
	
	frame.appendChild(ipAddress);
	frame.appendChild(upFor);
}

function restoreDefaultPreferences() {
	for (p in preferences) {
		var pref = eval('preferences.' + p);
		if (pref instanceof DOMPrefElement) {
			pref.value = pref.defaultValue;
		}
	}
	setPrefs();
}

function setPrefs() {
	body1.opacity = preferences.bgOpacity.value;
	body2.opacity = Math.round(preferences.bgOpacity.value / 3);
	body1.colorize = preferences.bgColor.value;
	body2.colorize = preferences.bgColor.value;
	
	cpu.font = preferences.fontFamily.value;
	cpu.color = preferences.textColor.value;
	cpu.size = preferences.fontSize.value;
	cpu.style.fontWeight = preferences.fontWeight.value;

	cpuSpeedShadow.font = preferences.fontFamily.value;
	cpuSpeedShadow.color = "#000000";
	cpuSpeedShadow.size = preferences.fontSize.value;
	cpuSpeedShadow.style.fontWeight = preferences.fontWeight.value;
	cpuSpeed.font = preferences.fontFamily.value;
	cpuSpeed.color = preferences.graphTextColor.value;
	cpuSpeed.size = preferences.fontSize.value;
	cpuSpeed.style.fontWeight = preferences.fontWeight.value;
	
	ram.font = preferences.fontFamily.value;
	ram.color = preferences.textColor.value;
	ram.size = preferences.fontSize.value;
	ram.style.fontWeight = preferences.fontWeight.value;

	netIn.font = preferences.fontFamily.value;
	netIn.color = preferences.textColor.value;
	netIn.size = preferences.fontSize.value;
	netIn.style.fontWeight = preferences.fontWeight.value;

	netOut.font = preferences.fontFamily.value;
	netOut.color = preferences.textColor.value;
	netOut.size = preferences.fontSize.value;
	netOut.style.fontWeight = preferences.fontWeight.value;

	netInTotalShadow.font = preferences.fontFamily.value;
	netInTotalShadow.color = "#000000";
	netInTotalShadow.size = preferences.fontSize.value;
	netInTotalShadow.style.fontWeight = preferences.fontWeight.value;
	netInTotal.font = preferences.fontFamily.value;
	netInTotal.color = preferences.graphTextColor.value;
	netInTotal.size = preferences.fontSize.value;
	netInTotal.style.fontWeight = preferences.fontWeight.value;

	netOutTotalShadow.font = preferences.fontFamily.value;
	netOutTotalShadow.color = "#000000";
	netOutTotalShadow.size = preferences.fontSize.value;
	netOutTotalShadow.style.fontWeight = preferences.fontWeight.value;
	netOutTotal.font = preferences.fontFamily.value;
	netOutTotal.color = preferences.graphTextColor.value;
	netOutTotal.size = preferences.fontSize.value;
	netOutTotal.style.fontWeight = preferences.fontWeight.value;
	
	upFor.font = preferences.fontFamily.value;
	upFor.color = preferences.textColor.value;
	upFor.size = preferences.fontSize.value;
	upFor.style.fontWeight = preferences.fontWeight.value;

	usedRamShadow.font = preferences.fontFamily.value;
	usedRamShadow.color = "#000000";
	usedRamShadow.size = preferences.fontSize.value;
	usedRamShadow.style.fontWeight = preferences.fontWeight.value;
	usedRam.font = preferences.fontFamily.value;
	usedRam.color = preferences.graphTextColor.value;
	usedRam.size = preferences.fontSize.value;
	usedRam.style.fontWeight = preferences.fontWeight.value;

	ipAddress.font = preferences.fontFamily.value;
	ipAddress.color = preferences.textColor.value;
	ipAddress.size = preferences.fontSize.value;
	ipAddress.style.fontWeight = preferences.fontWeight.value;

	var color1 = preferences.cpuGraphColor.value;
	var color2 = preferences.ramGraphColor.value;
	var color3 = preferences.netInGraphColor.value;
	var color4 = preferences.netOutGraphColor.value;
	cpuGraph.setColor(color1);
	ramGraph.setColor(color2);
	netInGraph.setColor(color3);
	netOutGraph.setColor(color4);
	
	if (parseInt(preferences.inBandwidth.value) <= 1 || isNaN(preferences.inBandwidth.value)) {
		preferences.inBandwidth.value = 320;
		alert("Invalid download bandwidth preference! Setting default value.");
	}
	
	if (parseInt(preferences.outBandwidth.value) <= 1 || isNaN(preferences.outBandwidth.value)){
		preferences.outBandwidth.value = 80;
		alert("Invalid upload bandwidth preference! Setting default value.");
	}
	
	updateIp();
	
}

function validatePlatform() {
	if (system.platform != 'windows') {
		alert("SystemMonitor is a Windows only widget, sorry.");
		closeWidget();
	}
}

function createServerCOM() {
	try {
		WMI = COM.createObject("WbemScripting.SWbemLocator");
		server = WMI.ConnectServer();
	} catch (e) {
		print("Failed to connect WMI server", e);
	}
}

function createSimpleMonitorCOM() {
	try {
		simpleMonitor = COM.createObject("SimpleMonitor.Information");
	} catch (e) {
		runCommand('"' + resolvePath("resources/exe/SimpleMonitor.exe") + '" /regserver');
		try {
			simpleMonitor = COM.createObject("SimpleMonitor.Information");
		} catch (e) {
			alert("Failed to create COM object, the widget will exit.");
			closeWidget();
		}
	}
}

function update() {
	try {
		trafficIn = simpleMonitor.trafficIn;
		trafficOut = simpleMonitor.trafficOut;
		uptime = simpleMonitor.uptime;
	} catch (e) {
		alert("SimpleMonitor failed to get vital data from back-end, the widget will exit.");
		closeWidget();
	}
	
	updateNetworkGraphs();
	updateSystemGraphs();
	updateUptime();
}

function updateCpuInfo() {
	updateCpuInfo(false);
}

function updateCpuInfo(full) {
	try {
		if (full) {
			// first call
			win = server.ExecQuery('SELECT CurrentClockSpeed, MaxClockSpeed, Name FROM Win32_Processor where DeviceId="CPU0"');
			processorSpeed = win.Item('Win32_Processor="CPU0"').CurrentClockSpeed;
			processorName  = win.Item('Win32_Processor="CPU0"').Name;
			processorMaxSpeed = win.Item('Win32_Processor="CPU0"').MaxClockSpeed;
		} else {
			// timer call
			win = server.ExecQuery('SELECT CurrentClockSpeed FROM Win32_Processor where DeviceId="CPU0"');
			processorSpeed = win.Item('Win32_Processor="CPU0"').CurrentClockSpeed;
		}
	} catch (e) {
		print(e);
	}
}

function updateSystemGraphs() {
	var cpuLoad = system.cpu.activity;
	var memLoad = system.memory.load;
	var usedRamHumanReadable = bytesToUIString(system.memory.totalPhysical - system.memory.availPhysical);
	var totalRamHumanReadable = bytesToUIString(system.memory.totalPhysical);
	var ramString = usedRamHumanReadable + " / " + totalRamHumanReadable;
	
	cpu.data = "CPU: " + cpuLoad + "%";
	cpuSpeed.data = processorSpeed && processorMaxSpeed ? processorSpeed + " / " + processorMaxSpeed + " MHz" : "";
	cpuSpeedShadow.data = cpuSpeed.data;
	cpuPanel.tooltip = processorName ? processorName : "";
	
	ram.data = "RAM: " + memLoad + "%";
	usedRam.data = ramString;
	usedRamShadow.data = ramString;
	ramPanel.tooltip = "Used: " + usedRamHumanReadable + " / Total: " + totalRamHumanReadable;

	cpuGraph.update(cpuLoad);
	ramGraph.update(memLoad);
}

function updateNetworkGraphs() {
	var totInString;
	var totOutString;
	var speedIn = 0;
	var speedOut = 0;
	
	if (trafficIn > 0 && trafficOut > 0) {
		if (lastIn != 0 && lastOut != 0) {
			speedIn = Math.round(trafficIn - lastIn);
			speedOut = Math.round(trafficOut - lastOut);
		}
		
		if (lastIn > trafficIn) {
			longIn++;
		}
		if (lastOut > trafficOut) {
			longOut++;
		}
		lastIn = trafficIn;
		lastOut = trafficOut;
		
		

		var currentIn = bytesToUIString(speedIn);
		var currentOut = bytesToUIString(speedOut);
		
		netIn.data = speedIn < 1024 ?  "In: 0 KB/s" : "In: " + currentIn + "/s";
		netOut.data = speedOut < 1024 ? "Out: 0 KB/s" : netOut.data = "Out: " + currentOut + "/s";
		
		totInString = "Total: " + bytesToUIString(trafficIn + (4294967295 * longIn));
		totOutString = "Total: " + bytesToUIString(trafficOut + (4294967295 * longOut));
		
		netInTotal.data = totInString;
		netOutTotal.data = totOutString;
		netInTotalShadow.data = totInString;
		netOutTotalShadow.data = totOutString;
		
		netInGraph.update(100 * speedIn / 1024 / preferences.inBandwidth.value);
		netOutGraph.update(100 * speedOut / 1024 / preferences.outBandwidth.value);
	} else {
		netIn.data = "Unknown";
		netOut.data = "Unknown";
		netInTotal.data = "";
		netOutTotal.data = "";
		netInTotalShadow.data = "";
		netOutTotalShadow.data = "";
	}
}

function updateUptime() {
	if (uptime) {
		upFor.data = uptime
	} else {
		upFor.data = "Failed getting uptime";
	}
}

function updateIp() {
	ipCheckRetry = 0;
	ipAddress.data = "...";
	preferences.showLocalIp.value == 1 ? fetchLocalIp() : fetchExternalIp();
}

// TODO: all this bullshit needs heavy refactoring... 
// create separate objects for all needed functionality

function fetchExternalIp() {
	if (ipCheckRetry >= ipCheckUrls.length) {
		return;
	}
	
	try {
		var request = new XMLHttpRequest();
		request.onreadystatechange = function() {
	        if (request.readyState == 4 && request.status == 200) { 
				var ip = this.responseText.match(/\b\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}\b/);
				if (ip) {
					ipAddress.data = "IP: " + ip;
					return;
				} else {
					fetchExternalIp();
				}
	        } else if (request.readyState == 4 && request.status != 200) {
				fetchExternalIp();
			}
		};
		request.open("GET", ipCheckUrls[ipCheckRetry++], true);
		request.send();
	} catch (e) {
		print(e);
	}
}

function fetchLocalIp() {
	set = server.ExecQuery("SELECT IPAddress FROM Win32_NetworkAdapterConfiguration where IPEnabled=True");
	// try to get the local IP (not the nicest way...)
	for (var i = 0; i <= 100; i++) {
		try {
			vbArray = set.Item('Win32_NetworkAdapterConfiguration=' + i).IPAddress;

			for (j = 0; j < vbArray.dimensions(); j++) {
				ip = vbArray.getItem(j);
				if (ip != null && ip.indexOf("192") != -1 || ip.indexOf("10") != -1) {
					ipAddress.data = "IP: " + ip;
					return;
				}
			}
		} catch (ex) {
			continue;
		}
	}
	return null;
}

function showContextMenu() {
	var name;
	var total;
	var free;
	var menu = new Array();
	var vols = filesystem.volumes;
	
	var restorePrefs = new MenuItem();
	restorePrefs.title = "Restore default preferences";
	restorePrefs.onSelect = function() {restoreDefaultPreferences();};
	menu.push(restorePrefs);
	
	var sep = new MenuItem();
	sep.title = "-";
	menu.push(sep);
	
	var styles = new MenuItem();
	styles.title = "Fast style";
	menu.push(styles);

	for (p in simpleMonitorPresets) {
		var preset = new MenuItem();
		preset.title = p;
		preset.onSelect = changeStylePreset(p);
		styles.appendChild(preset);
	}

	menu.push(sep);
	
	for (a in vols) {
		// ignore floppy drives...
		var path = vols[a].path;
		if (path.charAt(0) == "A" || path.charAt(0) == "B") {
			continue;
		}
		
		name = filesystem.getDisplayName(path);
		total = bytesToUIString(vols[a].totalBytes);
		free = bytesToUIString(vols[a].freeBytes);

		var item = new MenuItem();
		item.title = name + "\t[ " + free + " / "  + total + " ]";
		item.onSelect = openDrive(path);
		menu.push(item);
	}
	
	smWindow.contextMenuItems = menu;
}

function openDrive(path) {
	return function() {
		return filesystem.open(path);
	};
}

function changeStylePreset(preset) {
	return function() {
		preset = eval("simpleMonitorPresets." + preset);
		preferences.bgOpacity.value = preset.bgOpacity;
		preferences.bgColor.value = preset.bgColor;
		preferences.fontFamily.value = preset.fontFamily;
		preferences.textColor.value = preset.textColor;
		preferences.fontSize.value = preset.fontSize;
		preferences.fontWeight.value = preset.fontWeight;
		preferences.graphTextColor.value = preset.graphTextColor;
		preferences.cpuGraphColor.value = preset.cpuGraphColor;
		preferences.ramGraphColor.value = preset.ramGraphColor;
		preferences.netInGraphColor.value = preset.netInGraphColor;
		preferences.netOutGraphColor.value = preset.netOutGraphColor;
		setPrefs();
	};
}

